"""This module contains tests for the integration function of PySim
"""

from unittest import TestCase
import numpy as np
from numpy import cos, sin, sqrt

from pysim.simulation import Sim
from pysim.simulation import Runge_Kutta_4
from pysim.simulation import Cash_Karp
from pysim.simulation import Dormand_Prince_5

from pysim.systems import MassSpringDamper

from pysim.systems.python_systems import MassSpringDamper as PyMassSpringDamper

__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

def solve_msd_analytically(t):
    """Returns the analytical solution to the differential equation
    at the timepoints t.
    """
    mass = 100
    springk = 50
    damping = 1
    x0 = 1

    #Calculate help variables
    zeta = damping/(2*sqrt(mass*springk))
    omega0 = sqrt(springk/mass)

    #Solve the differential equation assuming zeta<0, an
    #under dampened system.
    assert zeta < 1
    omegad = omega0*sqrt(1-zeta**2)
    A = x0
    B = (zeta*omega0*x0)/omegad

    y = np.power(np.e, -1*zeta*omega0*t)*(A*cos(omegad*t)+B*sin(omegad*t))
    return y

class IntegrationTest(TestCase):
    """Tests the integration function of a under dampened mass-spring-damper
    system. The differential equation is solved analytically for the system
    and this solution is then compared to the PySim solution.
    """

    def setUp(self):
        self.sim = Sim()
        self.sys = MassSpringDamper()
        self.sys.store("x1")
        self.sim.add_system(self.sys)
        self.integrationlength = 50
        self.timestep = 0.1

    def compare_to_analytical(self, tolerance):
        """Compare the result in the sys to the analytical solution of the
        diff-equation. If the difference is larger than tolerance an assert
        will fail.
        """
        time = self.sys.res.time
        integrated_result = self.sys.res.x1
        analytic_result = solve_msd_analytically(time)

        maxerror = np.max(np.abs(integrated_result-analytic_result))
        print(maxerror)
        assert maxerror < tolerance



    def test_default(self):
        """Test the default sover is a Runge Kutta and compares the results
        solver results for accuracy
        """
        self.sim.simulate(self.integrationlength, self.timestep)
        self.compare_to_analytical(1e-5)

    def test_rk4(self):
        """Test the Runge Kutta solver results for accuracy"""
        solver = Runge_Kutta_4
        self.sim.simulate(self.integrationlength, self.timestep, solver)
        self.compare_to_analytical(1e-5)

    def test_dp5(self):
        """Test the Dormand Price solver results for accuracy when the
        default dense output is used, as well as the default relative
        error of 10^-4
        """
        solver = Dormand_Prince_5()
        self.sim.simulate(self.integrationlength, self.timestep, solver)
        self.compare_to_analytical(1e-3)

    def test_dp5_high_accuracy(self):
        """Test the Dormand Price solver results for accuracy with an increased
        relative error control of 10^-6. The default dense output is used.
        """
        solver = Dormand_Prince_5()
        solver.relative_error = 1e-6
        self.sim.simulate(self.integrationlength, self.timestep, solver)
        self.compare_to_analytical(1e-5)

    def test_dp5_dense_timediff(self):
        """Test that the timesteps delivered from the Dormand Price solver is
        regurarly spaced.
        """
        solver = Dormand_Prince_5()
        self.sim.simulate(self.integrationlength, self.timestep, solver)
        diff = np.diff(self.sys.res.time)
        absdifferr = np.abs(diff-self.timestep)
        assert np.max(absdifferr) < 1e-14


    def test_ck(self):
        """Test the Cash Karp solver results for accuracy with its default
        relative
        error of 10^-4
        """
        solver = Cash_Karp()
        self.sim.simulate(self.integrationlength, self.timestep, solver)
        self.compare_to_analytical(1e-2)

    def test_ck_high_accuracy(self):
        """Test the Cash Karp solver results for accuracy with an increased
        relative error control of 10^-6. The default dense output is used.
        """
        solver = Cash_Karp()
        solver.relative_error = 1e-8
        self.sim.simulate(self.integrationlength, self.timestep, solver)
        self.compare_to_analytical(1e-7)




class CythonIntegrationTest(IntegrationTest):
    """Use a Cython System to simulate. The results are compared with the
    analytical results.
    """

    def setUp(self):
        self.sim = Sim()
        self.sys = PyMassSpringDamper()
        self.sys.store("x1")
        self.sim.add_system(self.sys)
        self.integrationlength = 50
        self.timestep = 0.1



"""This module contains tests for for the parameter functionality
"""

from pysim.simulation import Sim
from pysim.systems import ParameterTestSystem

def get_system():
    """Get a default parameter test system and set it up"""
    sys = ParameterTestSystem()
    sys.pars.parameter_vector = [1,2,3]
    sys.pars.parameter_matrix = [[1,2,3],[4,5,6]]
    sys.pars.parameter_map = {'a':1, 'b':2}
    sys.pars.parameter_vectormap = {'a':[1,2], 'b':[3,4]}
    return sys

def test_vector_parameter():
    """Test that it is possible to set a vector parameter"""
    sys = get_system()
    assert sys.pars.parameter_vector == [1,2,3]
    sim = Sim()
    sim.add_system(sys)
    sim.simulate(0.1,0.1)
    assert sys.outputs.output_from_vector == 1

def test_matrix_parameter():
    """Test that it is possible to set a vector parameter"""
    sys = get_system()
    assert sys.pars.parameter_matrix == [[1,2,3],[4,5,6]]
    sim = Sim()
    sim.add_system(sys)
    sim.simulate(0.1,0.1)
    assert sys.outputs.output_from_matrix == 1

def test_map_parameter():
    """Test that it is possible to set a map parameter"""
    sys = get_system()
    assert sys.pars.parameter_map == {'a':1, 'b':2}
    sim = Sim()
    sim.add_system(sys)
    sim.simulate(0.1,0.1)
    assert sys.outputs.output_from_map == 1

def test_vectormap_parameter():
    """Test that it is possible to set a vectormap parameter"""
    sys = get_system()
    assert sys.pars.parameter_vectormap == {'a':[1,2], 'b':[3,4]}
    sim = Sim()
    sim.add_system(sys)
    sim.simulate(0.1,0.1)
    assert sys.outputs.output_from_vectormap == 2

if __name__ == "__main__":
    test_map_parameter()



"""Tests the compositesystem
"""
import numpy as np
from numpy.testing import assert_array_almost_equal

from pysim.simulation import Sim
from pysim.systems import MassSpringDamper
from pysim.systems import SquareWave
from pysim.systems import InOutTestSystem

from pysim.compositesystem import CompositeSystem

class ControlledSpring(CompositeSystem):
    """Composite system created for testing
    The system is made up of a square wave system and a mass-spring-damper 
    system. The square wave is applied as a force acting on the mass,
    driving it up and down.
    """
    def __init__(self):
        wave_sys = SquareWave()
        wave_sys.inputs.amplitude = 50
        wave_sys.inputs.freq = 0.1
        self.add_subsystem(wave_sys,"wave_sys")

        msd = MassSpringDamper()
        msd.inputs.b = 80
        msd.inputs.m = 50
        msd.inputs.f = 0
        self.add_subsystem(msd,"msd")

        wave_sys.connections.add_connection("signal", msd, "f")

        self.add_input_port("amp","wave_sys","amplitude", "amplitude of wave")
        self.add_output_port("out","msd","x1", "position")
        self.add_output_port("signal","wave_sys","signal", "signal from wave")

class CompositeSpring(CompositeSystem):
    """Composite system for testing purposes
    The system contains only a mass spring damper subsystem connected to 
    ports.
    """
    def __init__(self):
        msd = MassSpringDamper()
        msd.inputs.b = 80
        msd.inputs.m = 50
        msd.inputs.f = 0
        self.add_subsystem(msd,"msd")
        self.add_port_in_scalar("force", 0, "force acting on mass")
        self.connect_port_in("force", msd, "f")
        self.add_output_port("position","msd","x1", "Position")

class CompositeSquareWave(CompositeSystem):
    """Composite system representating a square wave, used for testing."""
    def __init__(self):
        wave_sys = SquareWave()
        wave_sys.inputs.amplitude = 50
        wave_sys.inputs.freq = 0.1
        self.add_subsystem(wave_sys,"wave_sys")

        self.add_input_port("freq",
                            "wave_sys",
                            "freq",
                            "frequency of wave")
        self.add_input_port("amplitude",
                            "wave_sys",
                            "amplitude",
                            "amplitude of wave")
        self.add_output_port("signal",
                             "wave_sys",
                             "signal",
                             "signal from wave")

class CompositeTestSystem(CompositeSystem):
    """Composite system used for testing composite systems."""
    def __init__(self):
        self.new_subsystems = [InOutTestSystem(),
                           InOutTestSystem()]

        for i in range(len(self.new_subsystems)):
            self.add_subsystem(self.new_subsystems[i],"subsystem_{}".format(i))

            self.add_port_in_scalar("scalar_in_{}".format(i), -1, "input scalar {}".format(i))
            self.connect_port_in("scalar_in_{}".format(i), self.new_subsystems[i], "input_scalar")
            portoutname = "output_scalar_{}".format(i)
            self.add_port_out_scalar(portoutname,-1,"Test scalar output from composite system {}".format(i))
            self.connect_port_out(portoutname, self.new_subsystems[i], "input_output_scalar")

            self.add_port_in_vector("vector_in_{}".format(i), (1,2,3), "input vector {}".format(i))
            self.connect_port_in("vector_in_{}".format(i), self.new_subsystems[i], "input_vector")
            portoutname = "output_vector_{}".format(i)
            self.add_port_out_vector(portoutname,(-1, -1, -1),"Test vector output from composite system {}".format(i))
            self.connect_port_out(portoutname, self.new_subsystems[i], "input_output_vector")

            self.add_port_in_matrix("matrix_in_{}".format(i), ((1,2),(3,4)), "input matrix {}".format(i))
            self.connect_port_in("matrix_in_{}".format(i), self.new_subsystems[i], "input_matrix")
            portoutname = "output_matrix_{}".format(i)
            self.add_port_out_matrix(portoutname,((-1,-1),(-1,-1)),"Test matrix output from composite system {}".format(i))
            self.connect_port_out(portoutname, self.new_subsystems[i], "input_output_matrix")

def test_port_connections():
    """Test the port connections to and from subsystems"""
    cs = CompositeTestSystem()
    ref_scalar = 5.0
    ref_vector = (6.0, 7.0, 8.0)
    ref_matrix = ((9.0, 10.0),(11.0, 12.0))
    cs.inputs.scalar_in_0 = ref_scalar
    cs.inputs.vector_in_0 = ref_vector
    cs.inputs.matrix_in_0 = ref_matrix
    sim = Sim()
    sim.add_system(cs)
    sim.simulate(0.5, 0.1)

    assert cs.outputs.output_scalar_0 == ref_scalar
    assert_array_almost_equal(cs.outputs.output_vector_0, ref_vector, 18)
    assert_array_almost_equal(cs.outputs.output_matrix_0,ref_matrix, 18)

def test_connected_subsystems():
    """Test that subsystems can be connected"""

    cd = ControlledSpring()
    sim = Sim()
    sim.add_system(cd)

    sim.simulate(2, 0.1)
    assert np.abs(cd.outputs.out-0.32406429942202225) < 1e-10

def test_connection_from_composite():
    """Test that it is possible to connect from a composite system to an
    ordinary.
    """
    sim = Sim()

    msd = MassSpringDamper()
    msd.inputs.b = 80
    msd.inputs.m = 50
    msd.inputs.f = 0
    sim.add_system(msd)

    sw = CompositeSquareWave()
    sw.inputs.amplitude = 50
    sw.inputs.freq = 0.1
    sim.add_system(sw)

    sw.connections.add_connection("signal",msd,"f")
    sim.simulate(2, 0.1)
    assert np.abs(msd.states.x1 - 0.3240587706226495) < 1e-10
    
def test_connection_to_composite():
    """Test that it is possible to connect from an ordinary system to
    a composite
    """
    sim = Sim()

    msd = CompositeSpring()
    sim.add_system(msd)

    sw = SquareWave()
    sw.inputs.amplitude = 50
    sw.inputs.freq = 0.1
    sim.add_system(sw)

    sw.connections.add_connection("signal",msd,"force")
    sim.simulate(2, 0.1)
    assert np.abs(msd.outputs.position - 0.32406429942202225) < 1e-10


def test_system_store():
    """Test that it is possible to store the output from a composite system"""
    cd = CompositeSpring()
    sim = Sim()
    sim.add_system(cd)
    cd.store("position")

    sim.simulate(2, 0.1)

    assert np.abs(cd.res.position[5]-0.90459733332768599) < 1e-7
    assert np.abs(cd.res.position[-1]-0.32406429942202225) < 1e-7



if __name__ == "__main__":
    test_connected_subsystems()



import pytest

from pysim.systems import InOutTestSystem
from pysim.simulation import Sim

class InheritanceTester(InOutTestSystem):
    pass

def test_inheritance():
    """Test that it is possible to inherit from a cpp system"""
    sys = InheritanceTester()
    assert sys.states.state_scalar == 1.23


if __name__ == "__main__":
    test_inheritance()



"""Python Simulation package (PySim)
This package enables the user to create simulations based on partial
differential equations. The equations are defined in Systems and
solved in Simulations
"""
__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

__version__ = "2.2.0"

def test():
    """Runs all the tests for pysim"""
    import pytest

    try:
        return pytest.main("--pyargs pysim")
    except SystemExit as e:
        return e.code



"""Contains default systems that can be used in simulations
"""
# Copyright should be placed above imports
#pylint: disable=wrong-import-position
__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

#The pylint warning is temporarily disabled here, should be enabled once the
#structure for build in systems is in place
#pylint: disable=wildcard-import
from pysim.systems.defaultsystemcollection1 import *



"""This module supplies info used to build extensions to pysim with cython
"""

import os
import os.path

__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

def get_include():
    """Gets the include file dir for cython extions to pysim"""
    currentdir = os.path.dirname(os.path.realpath(__file__))
    includedir = os.path.join(currentdir, "include")
    return includedir

def get_library_dir():
    """Gets the directory containg the library files needed to build
    cython extensions to pysim"""
    currentdir = os.path.dirname(os.path.realpath(__file__))
    libdir = os.path.join(currentdir, "lib")
    return [libdir]

def get_libraries():
    """Gets the name of the libraries used when building cython
    extensions to pysim"""
    return ["cppsystemlib",]



"""Tests various aspects of the Sim object that is not tested in other
places
"""
import tempfile
import json
import numpy as np
import pytest

from pysim.simulation import Sim
from pysim.systems import VanDerPol
from pysim.systems import MassSpringDamper
from pysim.systems import DiscretePID
from pysim.systems import RigidBody
from pysim.systems import LogisticMap
from pysim.systems.python_systems import VanDerPol as PyVanDerPol

__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

@pytest.mark.parametrize("test_class",[VanDerPol,PyVanDerPol])
def test_gettime(test_class):
    """Test that the elapsed time is returned from the simulation"""
    sys = test_class()
    sim = Sim()
    sim.add_system(sys)
    integrationlength = 2.0
    assert sim.get_time() == 0.0
    sim.simulate(integrationlength, 0.1)
    assert sim.get_time() == integrationlength

@pytest.mark.parametrize("test_class",[VanDerPol,PyVanDerPol])
def test_store_config(test_class):
    """Test that it is possible to store the simulation to a file.
    In this test a temp file is used, it should be deleted automatically
    after the test.
    """
    sys = test_class()
    sim = Sim()
    sim.add_system(sys)
    sys.inputs.a = 1.234
    sys.store("x")

    file = tempfile.NamedTemporaryFile(delete=False)
    file.close()
    sim.save_config(file.name)

    file2 = open(file.name)
    simdict = json.load(file2)
    file2.close()
    assert simdict["systems"]["vanderpol"]["inputs"]["a"] == 1.234
    assert "x" in simdict["systems"]["vanderpol"]["stores"]

def test_load_config():
    """Tests the loading of a system configuration from file"""

    configstring = """{
    "systems": {
        "vanderpol1": {
            "inputs": {
                "a": 1.234,
                "b": 1.0
            },
            "module": "pysim.systems",
            "type": "VanDerPol"
        },
       "vanderpol2": {
            "inputs": {
                "a": 1.0,
                "b": 3.456
            },
            "module": "pysim.systems.python_systems",
            "type": "VanDerPol"
        }
    }
}
"""

    sim = Sim()
    file = tempfile.NamedTemporaryFile(delete=False, mode="w+")
    file.write(configstring)
    file.close()
    sim.load_config(file.name)
    assert sim.systems["vanderpol1"].inputs.a == 1.234
    assert sim.systems["vanderpol2"].inputs.b == 3.456

def test_connected_system():
    """Check that the time for stored values in a discrete system is
    regurarly spaced"""

    #Create Simulaton
    sim = Sim()

    #Create, setup and add system to simulation
    sys = MassSpringDamper()
    sys.store("x1")
    sys.inputs.b = 50
    sys.inputs.f = 0
    sim.add_system(sys)

    controlsys = DiscretePID()
    controlsys.inputs.refsig = 1.0
    controlsys.inputs.p = 1
    controlsys.inputs.plim = 400.0
    controlsys.inputs.i = 0
    controlsys.inputs.stepsize = 0.3
    controlsys.store("outsig")
    sim.add_system(controlsys)

    sys.connections.add_connection("x1", controlsys, "insig")
    sys.connections.add_connection("x2", controlsys, "dsig")
    controlsys.connections.add_connection("outsig", sys, "f")
    controlsys.inputs.d = 1

    sim.simulate(5, 0.1)

    assert np.max(np.abs(np.diff(controlsys.res.time))-0.1) < 1e-14
    assert np.max(np.abs(np.diff(sys.res.time))-0.1) < 1e-14

def test_multiple_simulationobject():
    """Tests that it is possible to run multiple instances of the Sim object
    and that the results stay the same."""
    sim = Sim()
    sys = MassSpringDamper()
    sys.store("x1")
    sys.inputs.b = 50
    sys.inputs.f = 0
    sim.add_system(sys)
    sim.simulate(5, 0.1)
    xref = sys.res.x1
    for dummy in range(60):
        #Create Simulaton
        sim = Sim()
        sys = MassSpringDamper()
        sys.store("x1")
        sys.inputs.b = 50
        sys.inputs.f = 0
        sim.add_system(sys)
        sim.simulate(5, 0.1)
        x = sys.res.x1
        assert np.all(xref == x)

def test_state_break_larger():
    """Stop the simulation once the value of a state is 
    larger than a preset value
    """
    sim = Sim()
    sys = VanDerPol()
    sys.add_break_greater("y",1.0)
    sim.add_system(sys)
    sim.simulate(20,0.01)

    #If correct the simulation should break at time 0.79
    assert sys.res.time[-1] == 0.79

def test_state_break_smaller():
    """Stop the simulation once the value of a state is 
    larger than a preset value
    """
    sim = Sim()
    sys = VanDerPol()
    sys.add_break_smaller("x",-1.0)
    sim.add_system(sys)
    sim.simulate(20,0.01)

    #If correct the simulation should break at time 2.52
    assert sys.res.time[-1] == 2.52

def test_boost_vector_states():
    """Perform a basic simulation of a system with boost vector states"""
    sim = Sim()
    sys = RigidBody()

    sys.store("position")

    sys.inputs.force =  [1.0,0.0,0.0]
    sys.inputs.mass = 1.0

    sim.add_system(sys)
    sim.simulate(20,0.01)

    pos = sys.res.position
    diff = np.abs(pos[-1,:]-[200,0,0])
    assert np.max(diff) <= 1

def test_discrete_system():
    """Test a discrete system to make sure the results
    are correct. The system tested is a logistical map
    system and it is compared to a function created in 
    this test which also gives the solution.
    """
    lm = LogisticMap()
    lm.inputs.r = 3.6
    lm.states.x = 0.5
    lm.store("x")
    sim = Sim()
    sim.add_system(lm)

    sim.simulate(10,0.1)
    x = [0.5]
    r = 3.6
    for dummy in range(9):
        x.append(r*x[-1]*(1-x[-1]))

    assert np.all(np.abs(lm.res.x[1::10]-x)<1e-18)



if __name__ == "__main__":
    test_store_config(VanDerPol)



"""This module contains tests for the store function of pysim
"""
import re

from pysim.systems import VanDerPol

__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

def test_setstate():
    """Test that it is possible to set and receive a state variable.
    """
    sys = VanDerPol()
    sys.states.x = 1.0
    assert sys.states.x == 1.0

def test_getstatedoc():
    """Tests that the documentation of a state is returned."""
    sys = VanDerPol()
    s = sys.states.__repr__()
    print(s)

    comparestr = "{:>10}  {:>8.3}  {}".format("x", "1.0", "Position coordinate")
    found = re.search(comparestr,s,re.MULTILINE)
    assert found



"""This module contains tests for the setting and reading of inputs and outputs
"""
import re

import pytest
import numpy as np

from pysim.simulation import Sim

import pysim.cythonsystem

from pysim.systems import Adder3D
from pysim.systems import Adder
from pysim.systems import VanDerPol
from pysim.systems import MassSpringDamper
from pysim.systems import ReadTextInput
from pysim.systems import InOutTestSystem

class PythonInOutTestSystem(pysim.cythonsystem.Sys):
    """Python representation of the cpp InOutTestSystem

    Used for testing that the cpp system behaves as the python system
    with regards to the input output handling
    """
    def __init__(self):
        self.add_input_scalar("input_scalar")
        self.add_input_vector("input_vector",3)
        self.add_input_matrix("input_matrix",3,3)

        self.add_state_scalar("state_scalar","der_scalar")
        self.add_state_vector("state_vector","der_vector", 3)
        self.add_state_matrix("state_matrix","der_matrix", 3, 3)

        self.add_output_scalar("input_output_scalar")
        self.add_output_vector("input_output_vector",3)
        self.add_output_matrix("input_output_matrix",3,3)
        self.add_output_scalar("state_output_scalar")
        self.add_output_vector("state_output_vector",3)
        self.add_output_matrix("state_output_matrix",3,3)

        self.inputs.input_scalar = 0.0
        self.inputs.input_vector = [0.0, 0.0, 0.0]
        self.inputs.input_matrix = np.zeros((3,3))

        self.outputs.input_output_scalar = 0.0
        self.outputs.input_output_vector = [0.0, 0.0, 0.0]
        self.outputs.input_output_matrix = np.zeros((3,3))
        self.outputs.state_output_scalar = 0.0
        self.outputs.state_output_vector = [0.0, 0.0, 0.0]
        self.outputs.state_output_matrix = np.zeros((3,3))

        self.states.state_scalar = 1.23
        self.states.state_vector = np.ones(3)*4.56
        self.states.state_matrix = np.ones((3,3))*7.89
        self.ders.der_scalar = 0
        self.ders.der_vector = np.zeros(3)
        self.ders.der_matrix = np.zeros((3,3))

    def do_step(self,dummy):
        """During a timestep we set the outputs to their respective inputs"""
        self.outputs.input_output_scalar = self.inputs.input_scalar
        self.outputs.input_output_vector = self.inputs.input_vector
        self.outputs.input_output_matrix = self.inputs.input_matrix
        self.outputs.state_output_scalar = self.states.state_scalar
        self.outputs.state_output_vector = self.states.state_vector
        self.outputs.state_output_matrix = self.inputs.state_matrix

class PythonAdder3D(pysim.cythonsystem.Sys):
    """Class used in testing, equivalent to the c++ Adder3D"""

    def __init__(self):
        self.add_input_vector("input1",3)
        self.add_input_vector("input2",3)
        self.inputs.input1 = [0.0, 0.0, 0.0]
        self.inputs.input2 = [0.0, 0.0, 0.0]
        self.add_output_vector("output1",3)
        self.outputs.output1 = [0.0 ,0.0 ,0.0]

    def do_step(self,dummy):
        """Perform a timestep, adding the both inputs to create the 
        output.
        """
        i1 = np.array(self.inputs.input1)
        i2 = np.array(self.inputs.input2)
        self.outputs.output1 = i1+i2

class PythonAdder(pysim.cythonsystem.Sys):
    """Class used in testing, equivalent to the c++ Adder"""

    def __init__(self):
        self.add_input_scalar("input1")
        self.add_input_scalar("input2")
        self.inputs.input1 = 0.0
        self.inputs.input2 = 0.0
        self.add_output_scalar("output1")
        self.outputs.output1 = 0.0

    def do_step(self,dummy):
        """Perform a timestep, adding the both inputs to create the 
        output.
        """
        self.outputs.output1 = self.inputs.input1 + self.inputs.input2

__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

@pytest.mark.parametrize("adder_class",[Adder3D,PythonAdder3D])
def test_invalid_inputname(adder_class):
    """Test that an AttributeError is thrown if asking for a parameter that
    does not exist
    """
    sys = adder_class()
    with pytest.raises(AttributeError):
        dummy = sys.inputs.xyxyxy

@pytest.mark.parametrize("adder_class",[Adder3D,PythonAdder3D])
def test_invalid_outputname(adder_class):
    """Test that an AttributeError is thrown if asking for a variable that
    does not exist
    """
    sys = adder_class()
    with pytest.raises(AttributeError):
        dummy = sys.inputs.xyxyxy

@pytest.mark.parametrize("adder_class",[Adder3D,PythonAdder3D])
def test_invalid_input_dimension(adder_class):
    """Test that an ValueError is thrown if the number of elements in the
    parameter vector does not correspond with the number of values in the
    vector that it is being set to.
    """
    sys = adder_class()
    array2d = np.array((0.0, 0.0))
    with pytest.raises(ValueError):
        sys.inputs.input1 = array2d

@pytest.mark.parametrize("adder_class",[Adder3D,PythonAdder3D])
def test_invalid_vectorinput_type(adder_class):
    """Test that an TypeError is thrown when trying to set a vector parameter
    to a scalar.
    """
    sys = adder_class()
    with pytest.raises(TypeError):
        sys.inputs.input1 = 1

@pytest.mark.parametrize("adder_class",[Adder,PythonAdder])
def test_invalid_input_type(adder_class):
    """Test that an TypeError is thrown when trying to set a scalar parameter
    to a vector value.
    """
    sys = adder_class()
    array2d = np.array((0.0, 0.0))
    with pytest.raises(TypeError):
        sys.inputs.input1 = array2d


@pytest.mark.parametrize("adder_class",[Adder3D,PythonAdder3D])
def test_par_init(adder_class):
    """Tests that the par arrays are accessible and equal to 0"""
    sys = adder_class()
    x = sys.inputs.input1
    refarray = np.array((0.0, 0.0, 0.0))
    assert np.array_equal(x, refarray)

@pytest.mark.parametrize("adder_class",[Adder3D,PythonAdder3D])
def test_input_array_change(adder_class):
    """Tests that it is possible to change input array"""
    sys = adder_class()
    sys.inputs.input1 = (1.0, 2.0, 3.0)
    refarray = np.array((1.0, 2.0, 3.0))
    x = sys.inputs.input1
    assert np.array_equal(x, refarray)

@pytest.mark.parametrize("test_class",[InOutTestSystem,PythonInOutTestSystem])
def test_input_matrix_change(test_class):
    """Tests that it is possible to change input array"""
    sys = test_class()
    refarray = ((1.0, 2.0, 3.0),(4.0, 5.0, 6.0),(7.0, 8.0, 9.0))
    sys.inputs.input_matrix = refarray
    x = sys.inputs.input_matrix
    assert np.array_equal(x, refarray)

@pytest.mark.parametrize("adder_class",[Adder,PythonAdder])
def test_input_scalar_change(adder_class):
    """Tests that it is possible to change input scalar"""
    sys = adder_class()
    testvalue = 1.234
    sys.inputs.input1 = testvalue
    x = sys.inputs.input1
    assert np.array_equal(x, testvalue)

def test_input_string_change():
    """Tests that it is possible to change a text input"""
    sys = ReadTextInput()
    testvalue = "hej.txt"
    sys.pars.filename = testvalue
    x = sys.pars.filename
    assert x == testvalue

def test_getinputdoc():
    """Tests that the documentation of an input is returned."""
    sys = VanDerPol()
    s = sys.inputs.__repr__()

    comparestr = "{:>10}  {:>8.3}  {}".format("b", "1.0", "Scaling coefficient")
    found = re.search(comparestr,s,re.MULTILINE)
    assert found

@pytest.mark.parametrize("adder_class",[Adder3D,PythonAdder3D])
def test_output_vector_init(adder_class):
    """Tests that the output arrays are accessible and equal to 0"""
    sys = adder_class()
    x = sys.outputs.output1
    refarray = np.array((0.0, 0.0, 0.0))
    assert np.array_equal(x, refarray)

@pytest.mark.parametrize("test_class",[InOutTestSystem,PythonInOutTestSystem])
def test_output_matrix_init(test_class):
    """Tests that the output matrices are accessible and equal to 0"""
    sys = test_class()
    x = sys.outputs.input_output_matrix
    refarray = np.zeros((3,3))
    assert np.array_equal(x, refarray)

@pytest.mark.parametrize("adder_class",[Adder,PythonAdder])
def test_output_scalar_init(adder_class):
    """Tests that the output scalars are accessible and equal to 0"""
    sys = adder_class()
    x = sys.outputs.output1
    assert np.array_equal(x, 0)

@pytest.mark.parametrize("adder_class",[Adder3D,PythonAdder3D])
def test_output_change(adder_class):
    """Tests that it is possible to change input array.
    To make sure that the input is actually used in the simulation
    this is done by setting a input, run a simulation and
    see that the output value has changed.
    """
    sys = adder_class()
    refarray1 = np.array((0.0, 0.0, 0.0))
    inputarray = (1.0, 2.0, 3.0)
    sys.inputs.input1 = inputarray
    x1 = sys.outputs.output1
    assert np.array_equal(x1, refarray1)
    sim = Sim()
    sim.add_system(sys)
    sim.simulate(1, 0.1)
    x2 = sys.outputs.output1
    assert np.array_equal(x2, inputarray)


@pytest.mark.parametrize("adder_class1,adder_class2",
                         [(Adder3D,Adder3D),
                          (PythonAdder3D,PythonAdder3D),
                          (Adder3D,PythonAdder3D),
                          (PythonAdder3D,Adder3D),
                         ])
def test_connected_adder_system(adder_class1,adder_class2):
    """Check that it is possible to connect systems to each other 
    with boost vector outputs/inputs"""
    sys1 = adder_class1()
    sys2 = adder_class2()
    sys1.inputs.input1 = [1,2,3]
    sys1.connections.add_connection("output1",sys2,"input1")
    sim = Sim()
    sim.add_system(sys1)
    sim.add_system(sys2)
    assert np.all(sys2.outputs.output1 == [0.0, 0.0, 0.0])
    sim.simulate(1,0.1)
    assert np.all(sys2.outputs.output1 == [1.0, 2.0, 3.0])

@pytest.mark.parametrize("test_class1,test_class2",
                         [(InOutTestSystem,InOutTestSystem),
                          (PythonInOutTestSystem,PythonInOutTestSystem),
                          (InOutTestSystem,PythonInOutTestSystem),
                          (PythonInOutTestSystem,InOutTestSystem),
                         ])
def test_connected_system(test_class1,test_class2):
    """Check that it is possible to connect systems to each other 
    with matrix outputs/inputs"""
    sys1 = test_class1()
    sys2 = test_class2()
    refarray =  [[1,2,3],[4,5,6],[7,8,9]]
    sys1.inputs.input_matrix = refarray
    sys1.connections.add_connection("input_output_matrix",sys2,"input_matrix")
    sim = Sim()
    sim.add_system(sys1)
    sim.add_system(sys2)
    assert np.all(sys2.outputs.input_output_matrix == np.zeros((3,3)))
    sim.simulate(1,0.1)
    assert np.all(sys2.outputs.input_output_matrix == refarray)


@pytest.mark.parametrize("sys1_class,sys2_class",
                         [(InOutTestSystem,InOutTestSystem),
                          (PythonInOutTestSystem,PythonInOutTestSystem),
                          (InOutTestSystem,PythonInOutTestSystem),
                          (PythonInOutTestSystem,InOutTestSystem),
                         ])
def test_vector_scalar_conn(sys1_class,sys2_class):
    """Test that it is possible to connect vector to scalar

    An element of a vector output shall be possible to connect to an input
    scalar
    """
    sys1 = sys1_class()
    sys2 = sys2_class()
    sys3 = sys2_class()
    sys1.inputs.input_vector = [1,2,3]
    sys1.states.state_vector = [4,5,6]
    sys1.connections.add_connection("input_output_vector",sys2,"input_scalar",1)
    sys1.connections.add_connection("state_vector",sys3,"input_scalar",1)
    sim = Sim()
    sim.add_system(sys1)
    sim.add_system(sys2)
    sim.add_system(sys3)
    sim.simulate(0.1,0.1)
    assert sys2.outputs.input_output_scalar == 2.0
    assert sys3.outputs.input_output_scalar == 5.0

def test_der_as_output():
    """Test that it is possible to connect a derivative as output
    
    The der output should behave as a normal output. That this is the case is
    tested by connecting two systems to each output (der/output) and compare
    them.
    
    """
    sys1 = MassSpringDamper()
    sys2 = InOutTestSystem()
    sys3 = InOutTestSystem()
    sys1.connections.add_connection("dx2",sys2,"input_scalar")
    sys1.connections.add_connection("acceleration",sys3,"input_scalar")

    sys2.store("input_output_scalar")
    sys3.store("input_output_scalar")

    sim = Sim()
    sim.add_system(sys1)
    sim.add_system(sys2)
    sim.add_system(sys3)

    sim.simulate(1,0.1)

    output_from_der = sys2.res.input_output_scalar
    output_from_output = sys3.res.input_output_scalar
    assert np.allclose(output_from_der, output_from_output)

if __name__ == "__main__":
    test_input_matrix_change(InOutTestSystem)



"""Test the system ReadTextInput
"""
import tempfile
import numpy as np

from pysim.simulation import Sim
from pysim.systems import ReadTextInput

__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

def test_read_inputfile():
    """Tests that the system for reading text file values are ok
    It performs the test by creating a temporary file, filling the file with
    a numpy matrix. The file is read by the ReadTextInput system and the
    stored values are compared to the initial matrix.
    """
    timearray = np.arange(0.0, 100.0, 0.1)
    valuearray = timearray**2
    outmatrix = np.column_stack([timearray, valuearray])

    file = tempfile.NamedTemporaryFile(delete=False)
    np.savetxt(file, outmatrix, '%.6e')
    file.close()

    read_sys = ReadTextInput()
    read_sys.store("column_1")
    read_sys.pars.filename = file.name
    sim = Sim()
    sim.add_system(read_sys, "InputSystem")

    sim.simulate(100, 0.1)

    out = read_sys.res.column_1
    assert np.max(np.abs(out[3]-valuearray[3])) < 1e-15

if __name__ == "__main__":
    test_read_inputfile()



"""Example systems created in Python
"""

from pysim.cythonsystem import Sys

class VanDerPol(Sys):
    """Simple example of a class representing a VanDerPol oscillator.
    """
    def __init__(self):
        self.add_state_scalar("x", "dx")
        self.add_state_scalar("y", "dy")
        self.add_input_scalar("a")
        self.add_input_scalar("b")
        self.inputs.a = 1.0
        self.inputs.b = 1.0
        self.states.x = 1.0
        self.states.y = 0.0

    def do_step(self,dummy):
        """Perform a timestep by implmenting the VanDerPol equations"""
        
        a = self.inputs.a
        b = self.inputs.b
        x = self.states.x
        y = self.states.y

        self.ders.dx = a*x*(b-y*y)-y
        self.ders.dy = x


class MassSpringDamper(Sys):
    """Simple class for testing the mass-spring-damper simulations with 
    a cython system"""

    def __init__(self):
        """Setup two states (one dimensional vectors for now). Initial 
        conditions are simular to those in the build in c++ system"""
        self.add_state_scalar("x1", "dx1")
        self.add_state_scalar("x2", "dx2")
        self.states.x1 = 1
        self.states.x2 = 0

    def do_step(self,dummy):
        """Perform a step using default constants, same as those in the 
        cpp system"""

        m = 100.0
        b = 1.0
        k = 50.0
        f = 0.0
        x1 = self.states.x1
        x2 = self.states.x2
        self.ders.dx1 = x2
        self.ders.dx2 =-k/m*x1-b/m*x2+1/m*f



__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

import setuptools
from numpy.distutils.core import setup
from numpy.distutils.misc_util import Configuration
from distutils.extension import Extension
from Cython.Build import cythonize
import sys
import numpy
import os

from pysim import __version__

config = Configuration()
config.add_include_dirs(['pysim/cppsource', numpy.get_include()])

extracompileargs = []
if sys.platform == "win32":
    config.add_include_dirs([os.environ['BOOST_ROOT'],os.environ['EIGEN_ROOT']])

elif sys.platform in ("linux","darwin"):
    extracompileargs.append("-std=c++11")
    extracompileargs.append("-std=c++14")
    extracompileargs.append("-I/usr/include/eigen3")

config.add_installed_library("cppsystemlib",
                    ['pysim/cppsource/CppSystem.cpp',
                     'pysim/cppsource/StoreHandler.cpp',
                     'pysim/cppsource/CommonSystemImpl.cpp',
                     'pysim/cppsource/Variable.cpp',
                     'pysim/cppsource/ConnectionHandler.cpp',
                    ],
                    build_info = {
                    "extra_compiler_args":extracompileargs,
                    "language":"c++"},
                    install_dir = "pysim/lib",
                    )

                                
extensions = [Extension("pysim.cppsystem",
                        ['pysim/cppsystem.pyx',],
                        language="c++",
                        extra_compile_args=extracompileargs,
                        libraries=["cppsystemlib",]
                        ),
              Extension("pysim.cythonsystem",
                        ['pysim/cythonsystem.pyx','pysim/cppsource/CythonSystemImpl.cpp'],
                        language="c++",
                        extra_compile_args=extracompileargs,
                        libraries=["cppsystemlib",]
                        ),
              Extension("pysim.compositesystem",
                        ['pysim/compositesystem.pyx','pysim/cppsource/CompositeSystemImpl.cpp'],
                        language="c++",
                        extra_compile_args=extracompileargs,
                        libraries=["cppsystemlib",]
                        ),
              Extension("pysim.simulatablesystem",
                        ['pysim/simulatablesystem.pyx'],
                        language="c++",
                        extra_compile_args=extracompileargs,
                        libraries=["cppsystemlib",]
                        ),
              Extension("pysim.connections",
                        ['pysim/connections.pyx'],
                        language="c++",
                        extra_compile_args=extracompileargs,
                        libraries=["cppsystemlib",]
                        ),
              Extension("pysim.commonsystem",
                        ['pysim/commonsystem.pyx'],
                        language="c++",
                        extra_compile_args=extracompileargs,
                        libraries=["cppsystemlib",]
                        ),
              Extension("pysim.simulation",
                        ['pysim/simulation.pyx', 'pysim/cppsource/CppSimulation.cpp'],
                        language="c++",
                        extra_compile_args=extracompileargs,
                        ),
              Extension("pysim.systems.defaultsystemcollection1",
                         ['pysim/systems/defaultsystemcollection1/defaultsystemcollection1.pyx',
                          'pysim/systems/defaultsystemcollection1/cppsource/factory.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/MassSpringDamper.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/PredatorPrey.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/VanDerPol.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/Adder3D.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/Adder.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/ScalarAdder.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/SquareWave.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/DiscretePID.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/ReadTextInput.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/RigidBody.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/LogisticMap.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/ParameterTestSystem.cpp',
                          'pysim/systems/defaultsystemcollection1/cppsource/InOutTestSystem.cpp',
                          ],
                          language="c++",
                          extra_compile_args=extracompileargs,
                          include_dirs=['pysim/systems/defaultsystemcollection1',],
                          libraries=["cppsystemlib",],
                          ),
             ]

def readme():
    with open('README.rst') as f:
        return f.read()
setup(
    name="pysim",
    version=__version__,
    author="Linus Aldebjer",
    author_email="aldebjer@gmail.com",
    url="http://pys.im",
    ext_modules=cythonize(extensions),

    data_files=[('pysim/include',['pysim/cppsource/SimulatableSystem.hpp',
                                  'pysim/cppsource/CppSystem.hpp',
                                  'pysim/cppsource/PysimTypes.hpp',
                                  'pysim/cppsource/CommonSystemImpl.hpp',
                                  'pysim/cppsource/Variable.hpp',
                                  'pysim/cppsource/StoreHandler.hpp',
                                  'pysim/cppsource/CythonSystemImpl.hpp',
                                  'pysim/cppsource/ConnectionHandler.hpp',
                                  'pysim/cppsource/CompositeSystemImpl.hpp',
                                  ]),
                ('pysim',['pysim/cppsystem.pxd',
                          'pysim/commonsystem.pxd',
                          'pysim/simulatablesystem.pxd',
                          'pysim/cythonsystem.pxd',
                          'pysim/connections.pxd',
                          'pysim/compositesystem.pxd',
                          ]),
                ],
    packages=['pysim', 'pysim.systems','pysim.tests'],
    install_requires = ['numpy>=1.8.1',],
    description = "Modeling and Simulation of Dynamical Systems",
    long_description=readme(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    **config.todict()
)



"""This module contains tests for the store function of pysim
"""
import numpy as np

import pytest

from pysim.simulation import Sim
from pysim.systems import VanDerPol, SquareWave, RigidBody
from pysim.systems.python_systems import VanDerPol as PythonVanDerPol

__copyright__ = 'Copyright (c) 2014-2016 SSPA Sweden AB'

@pytest.mark.parametrize("test_class",[VanDerPol,PythonVanDerPol])
def test_store_state(test_class):
    """Test that it is possible to store a state"""
    sim = Sim()
    sys = test_class()
    sys.store("x")

    sim.add_system(sys)
    sim.simulate(10, 0.1)
    xres = sys.res.x
    assert isinstance(xres, np.ndarray)
    assert xres.size == 101

@pytest.mark.parametrize("test_class",[VanDerPol,PythonVanDerPol])
def test_store_der(test_class):
    """Test that it is possible to store a der"""
    sim = Sim()
    sys = test_class()
    sys.store("dx")

    sim.add_system(sys)
    sim.simulate(10, 0.1)
    dxres = sys.res.dx
    assert isinstance(dxres, np.ndarray)
    assert dxres.size == 101

@pytest.mark.parametrize("test_class",[VanDerPol,PythonVanDerPol])
def test_store_input(test_class):
    """Test that it is possible to store an input"""
    sim = Sim()
    sys = test_class()
    sys.store("a")

    sim.add_system(sys)
    sim.simulate(10, 0.1)
    ares = sys.res.a
    assert isinstance(ares,np.ndarray)
    assert ares.size == 101
    assert np.all(ares==1.0)

def test_store_output():
    """Test that it is possible to store a state"""
    sim = Sim()
    sys = SquareWave()
    sys.store("signal")

    sim.add_system(sys)
    sim.simulate(10, 0.1)
    res = sys.res.signal
    assert isinstance(res,np.ndarray)
    assert res.size == 101

def test_store_vector():
    """Test that it is possible to store a boost vector"""
    sim = Sim()
    sys = RigidBody()
    sys.store("position")
    sys.inputs.force =  [1,0,0]
    sim.add_system(sys)
    sim.simulate(20,0.01)
    diff = sys.res.position[-1,:]-sys.states.position
    assert np.all(diff == np.array([0.0, 0.0, 0.0]))

@pytest.mark.parametrize("test_class",[VanDerPol,PythonVanDerPol])
def test_store_after_added_system(test_class):
    """Tests that it is possible to first add a system to a simulation and
    then store parameters.
    """
    sim = Sim()
    sys = test_class()
    sim.add_system(sys)
    sys.store("x")

    sim.simulate(10, 0.1)
    ressize = sys.res.x.size
    assert ressize == 101

@pytest.mark.parametrize("test_class",[VanDerPol,PythonVanDerPol])
def test_time_store(test_class):
    """Check that the time is stored, and that the stored values includes
    both the beginning and the end of the simulation"""
    sim = Sim()
    sys = test_class()
    sim.add_system(sys)

    sim.simulate(2, 0.1)
    reftime = np.linspace(0,2,21)
    simtime = sys.res.time
    assert np.all(np.abs(simtime-reftime) <= np.finfo(float).eps)

@pytest.mark.parametrize("test_class",[VanDerPol,PythonVanDerPol])
def test_continue_store(test_class):
    """Test that it is possible to continue a simulation without storing
    values twice.
    """
    sim = Sim()
    sys = test_class()
    sim.add_system(sys)

    sys.store("x")
    sim.simulate(1, 0.1)
    sim.simulate(1, 0.1)
    reftime = np.linspace(0,2,21)
    simtime = sys.res.time
    assert np.all(np.abs(simtime-reftime) <= np.finfo(float).eps)

@pytest.mark.parametrize("test_class",[VanDerPol,PythonVanDerPol])
def test_interval_store(test_class):
    """Check that it is possible to change the interval between stored
    values.
    """
    sim = Sim()
    sys = test_class()
    sys.set_store_interval(0.2)
    sim.add_system(sys)

    sim.simulate(2, 0.1)
    reftime = np.linspace(0,2,11)
    simtime = sys.res.time
    assert np.all(np.abs(simtime-reftime) <= np.finfo(float).eps)
	
@pytest.mark.parametrize("test_class",[VanDerPol,PythonVanDerPol])
def test_midsim_store(test_class):
    """Check that it is possible to store a variable mid-simulation
    and that the result array is properly aligned with the timesteps
    and contains np.nan for locations where the variable was not stored.
    """
    sim = Sim()
    sys = test_class()
    
    sim.add_system(sys)
    sim.simulate(5, 1)
    sys.store("a")
    sim.simulate(5, 1)
    ares = sys.res.a
    assert np.all(np.isnan(ares[:6]))
    assert not np.any(np.isnan(ares[6:]))

